// Arc class -*- c++ -*-

#ifndef ARC_H_
# define ARC_H_
# ifdef __GNUC__
#  pragma interface
# endif // __GNUC__

/** @file Arc.h
 * Labelled arc in a net
 */

/** Arc between a Place and a Transition labelled with Marking expression */
class Arc
{
public:
  /** Constructor
   * @param output	Flag: is this an output arc?
   * @param expr	Arc expression (formal tokens)
   * @param place	Place associated with the arc
   */
  Arc (bool output,
       class Marking& expr,
       const class Place& place);
private:
  /** Copy constructor */
  Arc (const class Arc& old);
  /** Assignment operator */
  class Arc& operator= (const class Arc& old);
public:
  /** Destructor */
  ~Arc ();

  /** Determine whether this is an output arc */
  bool isOutput () const { return myIsOutput; }

  /** Get the arc expression */
  const class Marking& getExpr () const { return *myExpr; }
  /** Substitute the arc expression
   * @param substitution	the variable substitutions
   */
  void substitute (class Substitution& substitution);

  /** Get the place associated with this arc */
  const class Place& getPlace () const { return myPlace; }

  /** Get the index number of the place associated with this arc */
  unsigned getIndex () const { return myIndex; }

  /** Append an expression to the arc
   * @param expr       expression to be appended
   */
  void append (class Marking& expr);

private:
  /** Flag: is this an output arc? */
  bool myIsOutput;
  /** Arc expression (formal tokens) */
  class Marking* myExpr;
  /** Place associated with the Arc */
  const class Place& myPlace;
  /** Index number of the place in the net */
  const unsigned myIndex;
};

#endif // ARC_H_
