// Boolean type class -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "BoolType.h"
#include "LeafValue.h"
#include "Constraint.h"
#include "Range.h"

/** @file BoolType.C
 * Boolean data type
 */

class Value&
BoolType::getFirstValue () const
{
  const class Value* v = myConstraint ? &myConstraint->getFirstValue () : NULL;
  assert (!v || &v->getType () == this);
  return v
    ? *static_cast<class LeafValue*>(v->copy ())
    : *new class LeafValue (*this, false);
}

class Value&
BoolType::getLastValue () const
{
  const class Value* v = myConstraint ? &myConstraint->getLastValue () : NULL;
  assert (!v || &v->getType () == this);
  return v
    ? *static_cast<class LeafValue*>(v->copy ())
    : *new class LeafValue (*this, true);
}

card_t
BoolType::do_getNumValues () const
{
  assert (!myConstraint);
  return 2;
}

card_t
BoolType::convert (const class Value& value) const
{
  assert (value.getKind () == Value::vLeaf);
  if (myConstraint)
    return Type::convert (value);
  return bool (static_cast<const class LeafValue&>(value)) ? 1 : 0;
}

class Value*
BoolType::convert (card_t number) const
{
  assert (number < getNumValues ());
  if (myConstraint)
    return Type::convert (number);
  return new class LeafValue (*this, number ? true : false);
}

#ifdef EXPR_COMPILE
# include "StringBuffer.h"

void
BoolType::compileDefinition (class StringBuffer& out,
			     unsigned indent) const
{
  out.indent (indent);
  out.append ("bool_t");
}

void
BoolType::do_compileConversion (class StringBuffer& out,
				unsigned indent,
				const char* value,
				const char* number,
				bool add) const
{
  out.indent (indent);
  out.append (number);
  out.append (add ? "+=" : "=");
  out.append (value);
  out.append (";\n");
}

#endif // EXPR_COMPILE
